
class Exchange:

  def __init__(self, body):
    self.__body = body
    self.headers = {}

  def set_header(self, key, value):
    self.headers[key] = value
    return self

  @property
  def body(self):
    return self.__body

  @body.setter
  def body(self, value):
    self.set_body(value)

  def set_body(self, body):
    self.__body = body
    return self



class SourceWrapper:

  def __init__(self, source, route):
    self._source = source
    self.__route = route

  def wait_for_events(self):
    while True:
      exchange = self._source.wait_for_event()
      self.__route.process(exchange)

  def start(self):
    return self._source.start()


from ...framework.base.error_handler import ErrorHandler


class DeadLetterChannel(ErrorHandler):

  def __init__(self, endpoint):
    self.__endpoint = endpoint
    self.__prep_processor = None

  def on_prepare_failure(self, processor):
    self.__prep_processor = processor
    return self

  # todo hide this, move this
  def handle_exception(self, exchange):
    if self.__prep_processor:
      self.__prep_processor.process(exchange)
    self.__endpoint.process(exchange)







import abc

from ...framework.internal.route import Route
from ...component.error_handler.default_error_handler import DefaultErrorHandler


class RouteBuilder(metaclass=abc.ABCMeta):

  def __init__(self, *args, **kw):
    super().__init__(*args, **kw)
    self._routes = []
    self._error_handler = DefaultErrorHandler()

  def _from(self, source, route_id = None):
    route = Route(source, self._error_handler, route_id if route_id else str(self))
    self._routes.append(route)
    return route

  def _error_handler(self, error_handler):
    self._error_handler = error_handler

  @abc.abstractmethod
  def build(self):
    pass

  def get_routes(self):
    return self._routes

  def start(self):
    processes = []
    for route in self._routes:
      for process in route.start():
        processes.append(process)
    return process




import copy
import logging

from .exchange_handler import ExchangeHandler
from ...framework.base.error_handler import ErrorHandler
from .source_wrapper import SourceWrapper

logger = logging.getLogger(__name__)



class Route(object):

  def __init__(self, source, error_handler, route_id):
    self.__source = source
    self._source_wrapper = SourceWrapper(source, self)
    source.set_source_wrapper(self._source_wrapper)
    self._exchange_handlers = []
    self.__error_handler = error_handler
    self.__route_id = route_id

  @property
  def route_id(self):
    return self.__route_id

  # todo move this to separate class
  def process(self, exchange):
    try:
      next_exchange = copy.deepcopy(exchange)
      for exchange_handler in self._exchange_handlers:
        if exchange_handler.processor:
          exchange_handler.processor.process(next_exchange)
        elif exchange_handler.filter:
          keep_going = exchange_handler.filter.filter(next_exchange)
          if not keep_going:
            break
        elif exchange_handler.aggregate:
          next_exchange = exchange_handler.aggregate.aggregate(next_exchange)
          if not next_exchange:
            break
        else:
          raise Exception("Internal error: invalid exchange handler")
      self.__source.event_success(exchange)
    except Exception as err:
      try:
        exchange.set_header(ErrorHandler.EXCEPTION_CAUGHT, err)
        self.__source.event_failure(err, exchange)
        self.__error_handler.handle_exception(exchange)
      except Exception as err2:
        logger.error("Exception in error handler", exc_info=err2)
        logger.error("Original exception", exc_info=err)

  def to(self, processor):
    self._exchange_handlers.append(ExchangeHandler().set_processor(processor))
    return self

  def filter(self, filter):
    self._exchange_handlers.append(ExchangeHandler().set_filter(filter))
    return self

  def error_handler(self, error_handler):
    self.__error_handler = error_handler
    return self

  def start(self):
    return self._source_wrapper.start()

